"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fsStat = require("@nodelib/fs.stat");
const rpl = require("run-parallel");
function sync(root, options) {
    let names = options.fs.readdirSync(root);
    if (options.includeRootDirectory) {
        names = [root.toString()].concat(names);
    }
    const fsStatOptions = getFsStatOptions(options);
    const entries = [];
    for (const name of names) {
        const fullPath = path.join(root.toString(), name);
        if (options.preFilter && !options.preFilter(name, fullPath)) {
            continue;
        }
        const stats = fsStat.statSync(fullPath, fsStatOptions);
        const entry = makeDirEntry(name, fullPath, stats, options);
        if (options.filter && !options.filter(entry)) {
            continue;
        }
        entries.push(entry);
    }
    if (options.sort) {
        return entries.sort(options.sort);
    }
    return entries;
}
exports.sync = sync;
function async(root, options, callback) {
    const fsStatOptions = getFsStatOptions(options);
    options.fs.readdir(root, (err0, names) => {
        if (err0) {
            return callback(err0);
        }
        if (options.includeRootDirectory) {
            names = [root.toString()].concat(names); /* tslint:disable-line no-parameter-reassignment */
        }
        const preFilteredNames = [];
        const preFilteredPaths = [];
        const tasks = [];
        for (const name of names) {
            const fullPath = path.join(root.toString(), name);
            if (options.preFilter && !options.preFilter(name, fullPath)) {
                continue;
            }
            preFilteredNames.push(name);
            preFilteredPaths.push(fullPath);
            const task = (done) => fsStat.statCallback(fullPath, fsStatOptions, done);
            tasks.push(task);
        }
        rpl(tasks, (err1, stats) => {
            if (err1) {
                return callback(err1);
            }
            const entries = [];
            for (let index = 0; index < preFilteredNames.length; index++) {
                const name = preFilteredNames[index];
                const fullPath = preFilteredPaths[index];
                const stat = stats[index];
                const entry = makeDirEntry(name, fullPath, stat, options);
                if (options.filter && !options.filter(entry)) {
                    continue;
                }
                entries.push(entry);
            }
            if (options.sort) {
                return callback(null, entries.sort(options.sort));
            }
            callback(null, entries);
        });
    });
}
exports.async = async;
function getFsStatOptions(options) {
    return {
        fs: options.fs,
        followSymlinks: options.followSymlinks,
        throwErrorOnBrokenSymlinks: options.throwErrorOnBrokenSymlinks
    };
}
function makeDirEntry(name, full, stats, options) {
    const entry = {
        name,
        path: full,
        ino: stats.ino,
        isDirectory: stats.isDirectory(),
        isFile: stats.isFile(),
        isSymlink: stats.isSymbolicLink()
    };
    if (options.stats) {
        entry.stats = stats;
    }
    return entry;
}
exports.makeDirEntry = makeDirEntry;
